package de.fraunhofer.iem.dataprovider.toolResult

import de.fraunhofer.iem.dataprovider.gitlab.RepositoryRepository
import de.fraunhofer.iem.dataprovider.sarif.Sarif
import de.fraunhofer.iem.dataprovider.sarif.asDbObject
import org.springframework.stereotype.Service
import java.util.*

@Service
class ToolRunService(
    private val repositoryRepository: RepositoryRepository,
    private val toolRunRepository: ToolRunRepository,
    private val toolRepository: ToolRepository,
    private val ruleRepository: RuleRepository
) {

    // TODO: this func should be a suspend fun. We can achieve this by
    //  wrapping the db return types in flux, mono, or futures
    fun saveSarif(sarif: Sarif, repoId: UUID) {

        val repo = repositoryRepository.findById(repoId)

        sarif.runs.forEach { run ->
            val fullName = run.tool.driver.fullName
            val name = run.tool.driver.name
            val version = run.tool.driver.version

            val tool =
                toolRepository.findByFullNameIgnoreCaseAndNameIgnoreCaseAndVersionIgnoreCase(fullName, name, version)
                    ?: toolRepository.save(run.tool.asDbObject())

            val rules = if (tool.id != null) {
                ruleRepository.findByTool_Id(tool.id!!).toMutableList()
            } else {
                mutableListOf()
            }

            run.tool.driver.rules.forEach { sarifRule ->
                if (rules.find { it.sarifRuleId == sarifRule.id } == null) {
                    val rule = ruleRepository.save(sarifRule.asDbObject())
                    rules.add(rule)
                    tool.addRule(rule)
                }
            }

            val toolResults = run.results.map { sarifResult ->
                val tr = sarifResult.asDbObject()
                rules.filter { it.sarifRuleId == sarifResult.ruleId }
                    .forEach { it.addResultLocation(tr) }
                tr
            }


            val toolRun = run.asDbObject()
            toolRun.tool = tool
            toolRun.addRules(rules)
            toolRun.addToolResults(toolResults)

            val result = toolRunRepository.save(toolRun)
            toolRepository.save(tool)
            repo.ifPresent {
                it.addToolResult(result)
                repositoryRepository.save(it)
            }
        }
    }
}